with Ada.Text_IO, Messages, Reps, Parameters;
use Ada.Text_IO, Messages, Reps, Parameters;

with Numerics, Numerics.Ops, Numerics.IO;
use  Numerics, Numerics.Ops, Numerics.IO;

with Fpu;

procedure Branch is

-- arguments: Size PartSize filename

   package Txt_Int_IO is new Ada.Text_IO.Integer_IO(Integer);
   package Txt_Rep_IO is new Ada.Text_IO.Float_IO(Rep);

   subtype Scalar is Numeric;

   R1  :  constant Radius  := 0.1;            -- radius of analyticity

   Size:     constant Positive := GetArg(1);
   PartSize: constant Positive := GetArg(2);
   QName:    constant String   := GetArg(3);
   Eps :     constant Radius   := GetArg(4);
   D   :     constant Rep      := GetArg(5);  -- shift in frequency
   N   :     constant Integer  := GetArg(6);  -- number of shifts in frequency

   Number: Positive;
   Freq,InFreq: Rep;
   F: File_Type;

begin
   Open(F,In_File,QName);
   Txt_Rep_IO.Get(F,InFreq);
   Txt_Int_IO.Get(F,Number);
   Close(F);
   Sol :=  GetArg(7);
   if GetArg(8) = 1 then
      Display := True;
   end if;

   declare
      package INFpu is new Fpu (Number => Number, Size => Size, PartSize => PartSize, Scalar => Scalar);
      use INFpu, INFpu.Qf, INFpu.Qf_Ops, INFpu.Qf_IO;

      H: Q;

   begin
--      Trace;                        -- activate TraceEnter/TraceLeave
--      ShowParam;                    -- display some parameters

      Read(QName,1,H);
      SetRho(R1,H);
      Create(F,Out_File,QName & "-f");
      Close(F);
      for I in 0 .. N loop
         Freq := InFreq + Rep(I)*D;
         SetFrequency(Scal(Freq),H);
         Newton(Eps,H);
         Open(F,Append_File,QName & "-f");
         Txt_Rep_IO.Put(F,Freq);
         Txt_Rep_IO.Put(F,Approx(Norm2(H)));
         New_Line(F);
         Close(F);
      end loop;
   end;

end Branch;

