with Messages, Reps.Ops;
use Messages;

pragma Elaborate_All (Messages, Reps.Ops);

package body Fouriers.Ops is

  function IMin(I1,I2: Integer) return Integer renames Integer'Min;

  function "+"(H1,H2: Fourier) return Fourier is
     H : Fourier;
  begin
     if H1.J /= H2.J then
        Message("Fouriers.Ops.Add",Parity_Error);
     end if;
     H.J := H1.J;
     H.R := Radius'Min(H1.R,H2.R);
     for N in Freq loop
        H.C(N) := H1.C(N)+H2.C(N);
     end loop;
     if Trunc then return(H); end if;
     for N in FreqE loop
        H.E(N) := H1.E(N)+H2.E(N);
     end loop;
    return (H);
  end "+";

  function "-"(H1,H2: Fourier) return Fourier is
     H : Fourier;
  begin
     if H1.J /= H2.J then
        Message("Fouriers.Ops.Add",Parity_Error);
     end if;
     H.J := H1.J;
     H.R := Radius'Min(H1.R,H2.R);
     for N in Freq loop
        H.C(N) := H1.C(N)-H2.C(N);
     end loop;
     if Trunc then return(H); end if;
     for N in FreqE loop
        H.E(N) := H1.E(N)+H2.E(N);
     end loop;
    return (H);
  end "-";

  function "-"(H1: Fourier) return Fourier is
     H : Fourier;
  begin
     H.J := H1.J;
     H.R := H1.R;
     for N in Freq loop
        H.C(N) := -H1.C(N);
     end loop;
     if Trunc then return(H); end if;
     H.E := H1.E;
     return H;
  end "-";

  function "*"(S: Scalar; H: Fourier) return Fourier is
     H2 : Fourier;
   begin
     H2.J := H.J;
     H2.R := H.R;
     for N in Freq loop
        H2.C(N) := S*H.C(N);
     end loop;
     if Trunc then return(H2); end if;
     for N in FreqE loop
        H2.E(N) := SupAbs(S)*H.E(N);
     end loop;
    return (H2);
  end "*";

  function "*"(H1,H2: Fourier) return Fourier is
---  error estimates not optimal if H1.R /= H2.R
     J : constant Parity := (H1.J+H2.J) mod 2;
     R : constant Radius := Radius'Min(H1.R,H2.R);
  begin
     if IsZero(H1) then
        return (J,R,H1.C,H1.E);
     end if;

     declare
        M : Integer;
        S : Scalar;
        S1,S2 : Scalar := ScalHalf;
        W : ScalVec(FreqE);
        H : Fourier;
     begin
        if not Trunc then         -- Exp is slow, avoid frequent use
           W(0) := ScalOne;
           W(1) := Exp(Scal(R));
           M := 1;
           for K in 2 .. SizeE loop
              W(K) := W(K-M)*W(M);
              if 2*M=K then M := K; end if;
           end loop;
        end if;

        if H2.J=1 then
           if H1.J=0 then S1 := -S1; else S2 := -S2; end if;
        end if;

        SetZero(J,R,H);
        for K in Freq loop                 -- coeff*coeff
           for L in Freq loop
              S := H1.C(K)*H2.C(L);
              M := K+L;                    -- frequency sum
              if M <= Size then
                 H.C(M) := H.C(M)+S2*S;
              elsif not Trunc then
                 H.E(M) := H.E(M)+SupAbs(S)*W(M);
              end if;
              M := Abs(K-L);               -- frequency difference
              if J=1 and K<L then
                 H.C(M) := H.C(M)-S1*S;
              else
                 H.C(M) := H.C(M)+S1*S;
              end if;
           end loop;
        end loop;
        if Trunc then return(H); end if;

        for K in Freq loop                 -- coeff*error
           for L in FreqE loop
              S := ScalHalf*(SupAbs(H1.C(K))*H2.E(L)+SupAbs(H2.C(K))*H1.E(L));
              M := IMin(K+L,SizeE);        -- frequency sum
              H.E(M) := H.E(M)+S*W(K);
              M := L-K;                    -- frequency difference
              if M > 0 then
                 H.E(M) := H.E(M)+S/W(K);
              elsif L+M >= 0 then
                 H.E(J) := H.E(J)+S/W(L+M);
              else
                 H.E(J) := H.E(J)+S*W(-L-M);
              end if;
           end loop;
        end loop;

        for K in FreqE loop                 -- error*error
           for L in FreqE loop
              S := ScalHalf*H1.E(K)*H2.E(L);
              M := IMin(K+L,SizeE);         -- frequency sum
              H.E(M) := H.E(M)+S;
              M := IMin(K,L);               -- for frequency difference
              H.E(J) := H.E(J)+S/Sqr(W(M));
           end loop;
        end loop;

        if J=1 then
          H.C(0) := ScalZero;
          H.E(0) := ScalZero;
        end if;
        return (H);
     end;
  end "*";

  function Norm(H: Fourier) return Scalar is
     E : constant Scalar := Exp(Scal(H.R));
     W : Scalar := ScalOne;
     N : Scalar := ScalZero;
  begin
     for L in Freq loop
        N := N+SupAbs(H.C(L))*W;
        W := W*E;
     end loop;
     if Trunc then return N; end if;
     for L in FreqE loop
        N := N+H.E(L);
     end loop;
     return N;
  end Norm;

  function Norm2(H: Fourier) return Scalar is
     N : Scalar := ScalZero;
  begin
     for L in Freq loop
        N := N+Sqr(H.C(L));
     end loop;
     if Trunc then return N; end if;
--     for L in FreqE loop       TO BE FIXED
--        N := N+H.E(L);
--     end loop;
     return N;
  end Norm2;

  function MDm2(H: Fourier) return Fourier is
     F: Fourier;
  begin
     if H.J = 0 then
        Message("Fouriers.Ops.MDm2",Parity_Error);
     end if;
     F.J := 1;
     F.R := H.R;
     F.C(0) := ScalZero;
     for L in 1 .. Size loop
        F.C(L) := H.C(L)/Scal(L*L);
     end loop;
     if Trunc then return F; end if;
     F.E(0) := ScalZero;
     for L in 1 .. SizeE loop
        F.E(L) := H.E(L)/Scal(L*L);
     end loop;
     return F;
  end MDm2;

   procedure GuessRho(H: in Fourier; R: out Rep) is
     use Reps.Ops;
     Eps: constant := 1.0E-200;
     R1,R2: Rep := Zero;
  begin
     R := -Log(Eps);
     for L in Freq loop
        R1 := Rep'Max(R1,SupAbs(H.C(L)));
     end loop;
     if R1>Zero then
       for L in Size/2 .. Size loop
          R2 := -Log(SupAbs(H.C(L)/R1)+Eps)/Rep(L);
          R := Rep'Min(R,R2);
       end loop;
     end if;
  end GuessRho;

   function Antisymm(H: Fourier) return Fourier is
     F: Fourier;
  begin
     F.J := H.J;
     F.R := H.R;
     for L in Freq loop
        F.C(L) := Scal((-1)**L)*H.C(L);
     end loop;
     F.E := H.E;
     return F;
  end Antisymm;

   function Antisymm2(H: Fourier) return Fourier is
     F: Fourier;
  begin
     F.J := H.J;
     F.R := H.R;
     for L in Freq loop
        F.C(L) := Scal((-1)**(L+1))*H.C(L);
     end loop;
     F.E := H.E;
     return F;
  end Antisymm2;

end Fouriers.Ops;
