with Messages;
use Messages;


package body Fouriers is

   function Par(H: Fourier) return Parity is
   begin
      return H.J;
   end Par;

   function Rho(H: Fourier) return Radius is
   begin
      return H.R;
   end Rho;

   procedure SetRho(R: in Radius; H: in out Fourier) is
   begin
      H.R := R;
   end SetRho;

   procedure SetZero(J: in Parity; R: in Radius; H: out Fourier) is
   begin
      H.J := J;
      H.R := R;
      for N in Freq loop
         H.C(N) := ScalZero;
      end loop;
      if Trunc then return; end if;
      for N in FreqE loop
         H.E(N) := ScalZero;
      end loop;
   end SetZero;

   function IsZero(H: Fourier) return Boolean is
   begin
      for N in Freq loop
         if H.C(N) /= ScalZero then return False; end if;
      end loop;
      if Trunc then return True; end if;
      for N in FreqE loop
         if H.E(N) /= ScalZero then return False; end if;
      end loop;
      return True;
   end IsZero;

   function Component(N: Freq; H: Fourier) return Scalar is
   begin
      return H.C(N);
   end Component;

   function ErrComponent(N: FreqE; H: Fourier) return Scalar is
   begin
      if Trunc then return ScalZero; end if;
      return H.E(N);
   end ErrComponent;

   procedure SetComponent(N: Freq; S: in Scalar; H: in out Fourier) is
   begin
      if N<H.J and then S /= ScalZero then
        Show("S = ",Inf(S),Sup(S));
        Message("Fouriers.SetComponent",Index_Error);
      end if;
      H.C(N) := S;
   end SetComponent;

   procedure SetErrComponent(N: FreqE; S: in Scalar; H: in out Fourier) is
   begin
      H.E(Integer'Max(N,H.J)) := Abs(S);
   end SetErrComponent;

end Fouriers;
