with Messages, Reps.Ops, Linear.IO, Linear.Ops, Fpu_Packed;
use Messages, Reps.Ops, Linear.IO, Linear.Ops;

package body Fpu is

   use Qf_Ops, Qf_IO, Qf.Components, Qf_Ops.Components_Ops;

   ScalZero:  constant Scalar := Scal(Zero);
   ScalOne:   constant Scalar := Scal(One);
   Trunc: constant Boolean := IsNumeric(ScalZero);

   procedure SetSubComp(M: in Freq; N: in Part; S: in Scalar; H: in out Q) renames Qf.SetSubComponent;
   procedure SetErrComp(M: FreqE; N: Part; S: in Scalar; H: in out Q) renames Qf.SetErrComponent;
   pragma Inline (SetSubComp,SetErrComp);


   procedure ShowParam is
   begin
     if Trunc then
        Show("Running Numeric version");
     else
        Show("Running Interval version");
     end if;
     Show("  Number =",Number);
     Show("    Size =",Size);
     Show("PartSize =",PartSize);
   end ShowParam;

   function Contr(M: Matrix; H: Q) return Q is
      G: Q;
   begin
      G := F(H);
      return G-M*(G-H);
   end Contr;

   function DContr(M: Matrix; H,G: Q) return Q is
      F: Q;
   begin
      F := DF(H,G);
      return F-M*(F-G);
   end DContr;

   function DContr(M: Matrix; HM2,HP2,G: Q) return Q is
      F: Q;
   begin
      F := Basic.DF(HM2,HP2,G);
      return F-M*(F-G);
   end DContr;

   function DCNorm(M: Matrix; H: Q) return Scalar is
      MidSize: constant Integer := Integer'Min(PartSize+10,Size);
      NTest: Integer;
      S,E: Rep;
      S1,S2: Scalar;
      G1,G2,HM2,HP2: Q;
   begin
      TraceEnter("Fpu.DCNorm");
      G1 := DiffNeighM(H);
      HM2 := G1*G1;
      G2 := DiffNeighP(H);
      HP2 := G2*G2;

      S1 := ScalZero;
      for K1 in Part loop
         S2 := ScalZero;
         for K2 in Part loop
            S := Zero;
            NTest := MidSize;
            for N2 in Par(H) .. Size loop
               SetZero(Par(H),Rho(H),Frequency(H),G2);
               SetSubComp(N2,K2,ScalOne,G2);
               G1 := DContr(M,HM2,HP2,G2);
               E := Sup(Norm(Component(K1,G1))/Norm(Component(K2,G2)));
--               Show("from deg =" & N2'Img & " get",E);
               S := Rep'Max(S,E);
               if N2=NTest then                              -- test higher order ball
                  exit when Trunc;
                  SetZero(Par(H),Rho(H),Frequency(H),G2);
                  SetErrComp(N2+1,K2,ScalOne,G2);
                  G1 := DContr(M,HM2,HP2,G2);
                  E := Sup(Norm(Component(K1,G1)));
                  if E<S or N2=Size then                     -- done
                     Show("from deg >" & N2'Img & " get",E);
                     S := Rep'Max(S,E);
                     exit;
                  end if;
                  NTest := Integer'Min(NTest+10,Size);       -- for next test
               end if;
            end loop;
            Show("K1 =" & K1'Img & " K2 =" & K2'Img & " add",S);
            S2 := S2+Scal(S);
         end loop;
         Show("K1 =" & K1'Img & " totals",Sup(S2));
         S1 := Max(S1,S2);
      end loop;
      Show("Max over K1 is",Sup(S1));
      TraceLeave;
      return S1;
   end DCNorm;

   procedure CheckFix(M: in Matrix; H: in Q; R: in Radius) is
      Eps,K : Scalar;
   begin
      TraceEnter("Fpu.CheckFix");
      Eps := Norm(H-F(H));
      Show("||F(H)-H|| =",Sup(Eps));
      Eps := Norm(H-Contr(M,H));
--      Eps := Norm(M*(F(H)-H));
      Show("||C(H)-H|| =",Sup(Eps));
      K := DCNorm(M,H+Ball(Par(H),Rho(H),Frequency(H),R));
      Show(" ||DC(H)|| =",Sup(K));
      Show("R(1-K)-Eps =",Inf(R*(ScalOne-K)-Eps));
      TraceLeave;
   end CheckFix;

--- for numerical experiments

   package Packed is new Fpu_Packed (Number => Number, Size => Size, PartSize => PartSize);

   function DF(H: Q) return Matrix is
   begin
     if Trunc then return Basic.DF(H); end if;
     declare
         A: Vector(FullIndex);
      begin
         Pack(H,A);
         return Packed.DF(Par(H),Rho(H),Approx(Frequency(H)),A);
      end;
   end DF;

   function DFmI(H: Q) return Matrix is
   begin
      if Trunc then return Basic.DFmI(H); end if;
      declare
         A: Vector(FullIndex);
      begin
         Pack(H,A);
         return Packed.DFmI(Par(H),Rho(H),Approx(Frequency(H)),A);
      end;
   end DFmI;

   function Contr(H: Q) return Q is
   begin
      return Contr(DFmI(H),H);
   end Contr;

   procedure Newton(I: in Integer; H: in out Q) is
      A: Vector(FullIndex);
   begin
      TraceEnter("Fpu.Newton");
      Pack(H,A);
      Packed.Newton(I,Par(H),Rho(H),Approx(Frequency(H)),A);
      UnPack(A,Par(H),Rho(H),Frequency(H),H);
      Show("||F(H)-H|| =",Sup(Norm(F(H)-H)));
      Show("     ||H|| =",Sup(Norm(H)));
      TraceLeave;
   end Newton;

   procedure Newton(Eps: in Radius; H: in out Q) is
      A: Vector(FullIndex);
   begin
      TraceEnter("Fpu.Newton");
      Pack(H,A);
      Packed.Newton(Eps,Par(H),Rho(H),Approx(Frequency(H)),A);
      UnPack(A,Par(H),Rho(H),Frequency(H),H);
      Show("||F(H)-H|| =",Sup(Norm(F(H)-H)));
      Show("     ||H|| =",Sup(Norm(H)));
      TraceLeave;
   end Newton;

   procedure Newton(I: in Integer; M: in Matrix; H: in out Q) is
      A: Vector(FullIndex);
   begin
      TraceEnter("Fpu.Newton");
      Pack(H,A);
      Packed.Newton(I,Par(H),Rho(H),Approx(Frequency(H)),M,A);
      UnPack(A,Par(H),Rho(H),Frequency(H),H);
      Show("||F(H)-H|| =",Sup(Norm(F(H)-H)));
      Show("     ||H|| =",Sup(Norm(H)));
      TraceLeave;
   end Newton;

   procedure Shift(I,N: in Integer; D: in Scalar; H: in out Q) is
      Omega: Scalar;
   begin
      TraceEnter("Fpu.Shift");
      for J in 1 .. N loop
         Omega := Frequency(H)-D;
         SetFrequency(Omega,H);
         Newton(I,H);
      end loop;
      TraceLeave;
   end Shift;

end Fpu;
