with Ada.Text_IO, Reps, Linear, Fouriers_Init, Fpu_Basic;
use  Ada.Text_IO, Reps, Linear, Fouriers_Init;

generic

  Number,Size,PartSize: in Positive;
  type Scalar is private;

  with function IsNumeric(S: Scalar) return Boolean is <>;
  with function Scal(K: Integer) return Scalar is <>;
  with function Scal(R: Rep) return Scalar is <>;
  with function Ball(R: Rep) return Scalar is <>;
  with function Ball(S: Scalar) return Scalar is <>;
  with function Approx(S: Scalar) return Rep is <>;
  with function Inf(S: Scalar) return Rep is <>;
  with function Sup(S: Scalar) return Rep is <>;
  with function SupAbs(S: Scalar) return Rep is <>;
  with function "abs"(S: Scalar) return Scalar is <>;
  with function Min(S1,S2: Scalar) return Scalar is <>;
  with function Max(S1,S2: Scalar) return Scalar is <>;
  with function "-"(S: Scalar) return Scalar is <>;
  with function "+"(S,T: Scalar) return Scalar is <>;
  with function "-"(S,T: Scalar) return Scalar is <>;
  with function "*"(R: Rep; S: Scalar) return Scalar is <>;
  with function "/"(S: Scalar; R: Rep) return Scalar is <>;
  with function Sqr(S: Scalar) return Scalar is <>;
  with function "*"(S,T: Scalar) return Scalar is <>;
  with function Inv(S: Scalar) return Scalar is <>;
  with function "/"(S,T: Scalar) return Scalar is <>;
  with function "**"(S: Scalar; I: Integer) return Scalar is <>;
  with function Exp(S: Scalar) return Scalar is <>;
  with function Log(S: Scalar) return Scalar is <>;
  with procedure Show(S: in Scalar; NewLine: in Boolean := True) is <>;
  with procedure Get(F: in File_Type; S: out Scalar; Decimal: in Boolean := True) is <>;
  with procedure Put(F: in File_Type; S: in Scalar; Decimal: in Boolean := True) is <>;

package Fpu is

   package Basic is new Fpu_Basic (Number => Number, Size => Size, PartSize => PartSize, Scalar => Scalar);
   package Qf renames Basic.Qb;
   package Qf_Ops renames Basic.Qb_Ops;
   package Qf_IO renames Basic.Qb_IO;
   use Qf;

   subtype FullIndex is Basic.FullIndex;     -- vector index
   subtype PartIndex is Basic.PartIndex;     -- vector index

   procedure ShowParam;                      -- show some parameters

   function F(H: Q) return Q renames Basic.F;               -- F(H) := -D^(-2)(H+H^3)
   function DF(H,G: Q) return Q renames Basic.DF;           -- DF(H)G
   function Contr(M: Matrix; H: Q) return Q;                -- F(H)-M(F(H)-H)
   function DContr(M: Matrix; H,G: Q) return Q;             -- derivative of Contr
   function DCNorm(M: Matrix; H: Q) return Scalar;          -- norm of DContr
   procedure CheckFix(M: in Matrix; H: in Q; R: in Radius); -- check ...

--- for numerical experiments

   function DF(H: Q) return Matrix;                         -- approx DF(H)
   function DFmI(H: Q) return Matrix;                       -- approx (DF(H)-I)^(-1)+I
   function Contr(H: Q) return Q;                           -- F(H)-DFmI(H)(F(H)-H)
   procedure Newton(I: in Integer; H: in out Q);
   procedure Newton(Eps: in Radius; H: in out Q);
   procedure Newton(I: in Integer; M: in Matrix; H: in out Q);
   procedure Shift(I,N: in Integer; D: in Scalar; H: in out Q);

private

   function DContr(M: Matrix; HM2,HP2,G: Q) return Q;

end Fpu;
