with Messages, Reps.Ops, Linear.IO, Linear.Ops;
use Messages, Reps.Ops, Linear.IO, Linear.Ops;

package body Fpu_Basic is

   use Qb_IO, Qb_Ops, Qb.Components;

   function F(H: Q) return Q is
      S: constant Scalar := Inv(Frequency(H));
      GM,GP: Q;
   begin
      GM := DiffNeighM(H);
      GP := DiffNeighP(H);
      return Sqr(S)*MDm2(GM-GP+GM*GM*GM-GP*GP*GP);
   end F;

   function DF(H,G: Q) return Q is
      S: constant Scalar := Inv(Frequency(H));
      GM,GP,HM,HP: Q;
   begin
      GM := DiffNeighM(G);
      GP := DiffNeighP(G);
      HM := DiffNeighM(H);
      HP := DiffNeighP(H);
      return Sqr(S)*Mdm2(GM-GP+Scal(3)*((GM*HM)*HM-(GP*HP)*HP));
   end DF;

   function DF(HM2,HP2,G: Q) return Q is
     S: constant Scalar := Inv(Frequency(G));
     GM,GP: Q;
   begin
     GM := DiffNeighM(G);
     GP := DiffNeighP(G);
     return Sqr(S)*Mdm2(GM-GP+Scal(3)*(GM*HM2-GP*HP2));
   end DF;

--- for numerical experiments

   function DF(H: Q) return Matrix is
      pragma Suppress(Storage_Check);
      V : Vector(PartIndex);
      M : Matrix(PartIndex,PartIndex);
      G,HM2,HP2: Q;
   begin
      TraceEnter("Fpu_Basic.DF");
      G := DiffNeighM(H);
      HM2 := G*G;
      G := DiffNeighP(H);
      HP2 := G*G;

      SetZero(M);
      for I in Part loop
         for J in Par(H) .. PartSize loop
            SetZero(Par(H),Rho(H),Frequency(H),G);
            SetSubComponent(J,I,Scal(One),G);
            Pack(DF(HM2,HP2,G),V);
            SetColumn(Pack(J,I),V,M);
         end loop;
      end loop;
      TraceLeave;
      return(M);
   end DF;

   function DFmI(H: Q) return Matrix is
      pragma Suppress(Storage_Check);
      M: Matrix(PartIndex,PartIndex);
   begin
      TraceEnter("Fpu_Basic.DFmI");
      M := DF(H);
      Add(-One,M);
      Invert(M);
      Add(One,M);
      TraceLeave;
      return M;
   end DFmI;

end Fpu_Basic;
