with Ada.Text_IO, Reps, Linear, Fouriers_Init, Qs.IO, Qs.Ops;
use  Ada.Text_IO, Reps, Linear, Fouriers_Init;

generic

  Number,Size,PartSize: in Positive;
  type Scalar is private;

  with function IsNumeric(S: Scalar) return Boolean is <>;
  with function Scal(K: Integer) return Scalar is <>;
  with function Scal(R: Rep) return Scalar is <>;
  with function Ball(R: Rep) return Scalar is <>;
  with function Ball(S: Scalar) return Scalar is <>;
  with function Approx(S: Scalar) return Rep is <>;
  with function Inf(S: Scalar) return Rep is <>;
  with function Sup(S: Scalar) return Rep is <>;
  with function SupAbs(S: Scalar) return Rep is <>;
  with function "abs"(S: Scalar) return Scalar is <>;
  with function Min(S1,S2: Scalar) return Scalar is <>;
  with function Max(S1,S2: Scalar) return Scalar is <>;
  with function "-"(S: Scalar) return Scalar is <>;
  with function "+"(S,T: Scalar) return Scalar is <>;
  with function "-"(S,T: Scalar) return Scalar is <>;
  with function "*"(R: Rep; S: Scalar) return Scalar is <>;
  with function "/"(S: Scalar; R: Rep) return Scalar is <>;
  with function Sqr(S: Scalar) return Scalar is <>;
  with function "*"(S,T: Scalar) return Scalar is <>;
  with function Inv(S: Scalar) return Scalar is <>;
  with function "/"(S,T: Scalar) return Scalar is <>;
  with function "**"(S: Scalar; I: Integer) return Scalar is <>;
  with function Exp(S: Scalar) return Scalar is <>;
  with function Log(S: Scalar) return Scalar is <>;
  with procedure Show(S: in Scalar; NewLine: in Boolean := True) is <>;
  with procedure Get(F: in File_Type; S: out Scalar; Decimal: in Boolean := True) is <>;
  with procedure Put(F: in File_Type; S: in Scalar; Decimal: in Boolean := True) is <>;

package Fpu_Basic is

   package Qb is new Qs (Number => Number, Size => Size, Scalar => Scalar);
   package Qb_IO is new Qb.IO;
   package Qb_Ops is new Qb.Ops;
   use Qb;

   subtype FullIndex is Integer range 1 .. Number*Size;     -- vector index
   subtype PartIndex is Integer range 1 .. Number*PartSize; -- vector index

   function F(H: Q) return Q;           -- F(H) := -D^(-2)(H+H^3)
   function DF(H,G: Q) return Q;        -- DF(H)G
   function DF(HM2,HP2,G: Q) return Q;  -- DF(H)G  with HM2,HP2 already computed

--- for numerical experiments

   function DF(H: Q) return Matrix;     -- approx DF(H)
   function DFmI(H: Q) return Matrix;   -- approx (DF(H)-I)^(-1)+I

end Fpu_Basic;
