with Messages, Numerics, Numerics.Ops, Numerics.IO, Reps.Ops,Parameters;
use Messages, Numerics, Numerics.Ops, Numerics.IO,Parameters;
with Fpu_Basic;

package body Fpu_Packed is

   package Basic is new Fpu_Basic (Number => Number, Size => Size, PartSize => PartSize, Scalar => Numeric);
   use Basic, Basic.Qb, Basic.Qb_Ops, Basic.Qb_IO;

   function DF(J: Parity; R: Radius; Fr: Rep; A: Vector) return Matrix is
     H: Q;
   begin
     UnPack(A,J,R,Scal(Fr),H);
     return DF(H);
   end DF;

   function DFmI(J: Parity; R: Radius; Fr: Rep; A: Vector) return Matrix is
     H: Q;
   begin
     UnPack(A,J,R,Scal(Fr),H);
     return DFmI(H);
   end DFmI;

   procedure Newton(I: in Integer; J: in Parity; R: in Radius; Fr: in Rep; A: in out Vector) is
     G,H: Q;
   begin
     UnPack(A,J,R,Scal(Fr),H);
     for N in 1 .. I loop
       G := F(H);
       if Display then
          Show("|F(H)-H| =",Sup(Norm(G-H)));
       end if;
       H := G-DFmI(H)*(G-H);
     end loop;
     Pack(H,A);
   end Newton;

   procedure Newton(Eps: in Radius; J: in Parity; R: in Radius; Fr: in Rep; A: in out Vector) is
     pragma Suppress(Storage_Check);
     use Reps.Ops;
     Nmax: constant Integer := 32;
     Ratio: constant Rep := Half;
     E,Et: Rep := Eps;
     G,H: Q;
     M: Matrix(PartIndex,PartIndex);
   begin
     UnPack(A,J,R,Scal(Fr),H);
     for N in 1 .. Nmax loop
        G := F(H);
        Et := E;
        E := Sup(Norm(G-H));
        if Display then
           Show("|F(H)-H| =",E);
        end if;
        if E<Eps then                          -- done
           Pack(H,A);
           return;
        end if;
        if E/Et>Ratio or E>Sqrt(Eps) then      -- need to update M
           M := DFmI(H);
        end if;
        H := G-M*(G-H);
     end loop;
     Show("Fpu_Packed.Newton: giving up");
     Pack(H,A);
   end Newton;

   procedure Newton(I: in Integer; J: in Parity; R: in Radius; Fr: in Rep; M: in Matrix; A: in out Vector) is
     G,H: Q;
   begin
     UnPack(A,J,R,Scal(Fr),H);
     for N in 1 .. I loop
       G := F(H);
       Show("|F(H)-H| =",Sup(Norm(G-H)));
       H := G-M*(G-H);
     end loop;
     Pack(H,A);
   end Newton;

end Fpu_Packed;
