with Ada.Text_IO, Ada.Sequential_IO;

pragma Elaborate_All (Ada.Text_IO, Ada.Sequential_IO);

package body Linear.IO is

  package Txt_Int_IO is new Ada.Text_IO.Integer_IO(Integer);
  package Txt_Rep_IO is new Ada.Text_IO.Float_IO(Rep);
  package Seq_Rep_IO is new Ada.Sequential_IO(Rep);

  procedure ShortPut(F: in Ada.Text_IO.File_Type; I: in Integer) is
    K,D: Integer;
  begin
    K := Abs(I);
    if I<0 then D := 2; else D := 1; end if;
    while K>9 loop
      K := K/10;
      D := D+1;
    end loop;
    Txt_Int_IO.Put(F,I,D,10);
  end ShortPut;

  procedure BinRead(Name: in String; B: out Vector) is
    use Seq_Rep_IO;
    F: File_Type;
    R: Rep;
  begin
    Open(F,In_File,Name);
    for I in B'Range loop
      R := Zero;
      if not End_Of_File(F) then
        Read(F,R);
      end if;
      B(I) := R;
    end loop;
    Close(F);
  end BinRead;

  procedure TextRead(Name: in String; B: out Vector) is
    use Ada.Text_IO, Txt_Rep_IO;
    F: File_Type;
    R: Rep;
  begin
    Open(F,In_File,Name);
    for I in B'Range loop
      R := Zero;
      if not End_Of_File(F) then
        Get(F,R);
      end if;
      B(I) := R;
    end loop;
    Close(F);
  end TextRead;

  procedure Read(Name: in String; B: out Vector; Bin: in Boolean := False) is
    use Ada.Text_IO;
  begin
    Put_Line("reading " & Name);
    if Bin then
      BinRead(Name,B);
    else
      TextRead(Name,B);
    end if;
  end Read;

  procedure BinRead(Name: in String; A: out Matrix) is
    use Seq_Rep_IO;
    F: File_Type;
    R: Rep;
  begin
    Open(F,In_File,Name);
    for I in A'Range(1) loop
      for J in A'Range(2) loop
        R := Zero;
        if not End_Of_File(F) then
          Read(F,R);
        end if;
        A(I,J) := R;
      end loop;
    end loop;
    Close(F);
  end BinRead;

  procedure TextRead(Name: in String; A: out Matrix) is
    use Ada.Text_IO, Txt_Rep_IO;
    F: File_Type;
    R: Rep;
  begin
    Open(F,In_File,Name);
    for I in A'Range(1) loop
      for J in A'Range(2) loop
        R := Zero;
        if not End_Of_File(F) then
          Get(F,R);
        end if;
        A(I,J) := R;
      end loop;
    end loop;
    Close(F);
  end TextRead;

  procedure Read(Name: in String; A: out Matrix; Bin: in Boolean := False) is
    use Ada.Text_IO;
  begin
    Put_Line("reading " & Name);
    if Bin then
      BinRead(Name,A);
    else
      TextRead(Name,A);
    end if;
  end Read;

  procedure BinWrite(Name: in String; B: in Vector) is
    use Seq_Rep_IO;
    F: Seq_Rep_IO.File_Type;
  begin
    Create(F,Out_File,Name);
    for I in B'Range loop
      Write(F,B(I));
    end loop;
    Close(F);
  end BinWrite;

  procedure TextWrite(Name: in String; B: in Vector) is
    use Ada.Text_IO, Txt_Rep_IO;
    F: File_Type;
  begin
    Create(F,Out_File,Name);
    for I in B'Range loop
      Put(F,B(I),2,16,4);    -- File,Item,Fore,Aft,Exp
      New_Line(F);
    end loop;
    Close(F);
  end TextWrite;

  procedure Write(Name: in String; B: in Vector; Bin: in Boolean := False) is
    use Ada.Text_IO;
  begin
    Put_Line("writing " & Name);
    if Bin then
      BinWrite(Name,B);
    else
      TextWrite(Name,B);
    end if;
  end Write;

  procedure BinWrite(Name: in String; A: in Matrix) is
    use Seq_Rep_IO;
    F: Seq_Rep_IO.File_Type;
  begin
    Create(F,Out_File,Name);
    for I in A'Range(1) loop
      for J in A'Range(2) loop
        Write(F,A(I,J));
      end loop;
    end loop;
    Close(F);
  end BinWrite;

  procedure TextWrite(Name: in String; A: in Matrix) is
    use Ada.Text_IO, Txt_Rep_IO;
    F: File_Type;
  begin
    Create(F,Out_File,Name);
    for I in A'Range(1) loop
      for J in A'Range(2) loop
        Put(F,A(I,J),2,16,4);     -- File,Item,Fore,Aft,Exp
        New_Line(F);
      end loop;
    end loop;
    Close(F);
  end TextWrite;

  procedure Write(Name: in String; A: in Matrix; Bin: in Boolean := False) is
    use Ada.Text_IO;
  begin
    Put_Line("writing " & Name);
    if Bin then
      BinWrite(Name,A);
    else
      TextWrite(Name,A);
    end if;
  end Write;

  procedure Show(P: in Pivot; L: in Integer := Integer'Last) is
    use Ada.Text_IO;
  begin
    for I in P'First .. Integer'Min(P'Last,L) loop
      Put("P(");
      ShortPut(Current_Output,I);
      Put(") = ");
      ShortPut(Current_Output,P(I));
      New_Line;
    end loop;
    New_Line;
  end Show;

  procedure Show(B: in Vector; Cut: in Rep := Zero) is
    use Ada.Text_IO, Txt_Rep_IO;
  begin
    for I in B'Range loop
      if Abs(B(I))>Cut then
        Put("B(");
        ShortPut(Current_Output,I);
        Put(") = ");
        Put(Current_Output,B(I),2,16,4);        -- File,Item,Fore,Aft,Exp
        New_Line;
      end if;
    end loop;
    New_Line;
  end Show;

  procedure Show(B1,B2: in Vector; Cut: in Rep := Zero) is
    use Ada.Text_IO, Txt_Rep_IO;
  begin
    for I in B2'Range loop
      if Abs(B1(I))>Cut or Abs(B2(I))>Cut then
        Put("(");
        ShortPut(Current_Output,I);
        Put(")  ");
        Put(Current_Output,B1(I),2,16,4);        -- File,Item,Fore,Aft,Exp
        Put("   ");
        Put(Current_Output,B2(I),2,16,4);        -- File,Item,Fore,Aft,Exp
        New_Line;
      end if;
    end loop;
    New_Line;
  end Show;

  procedure Show(A: in Matrix; Cut: in Rep := Zero) is
    use Ada.Text_IO, Txt_Rep_IO;
  begin
    for I in A'Range(1) loop
      for J in A'Range(2) loop
        if Abs(A(I,J))>Cut then
          Put("A(");
          ShortPut(Current_Output,I);
          Put(",");
          ShortPut(Current_Output,J);
          Put(") = ");
          Put(Current_Output,A(I,J),2,16,4);        -- File,Item,Fore,Aft,Exp
          New_Line;
        end if;
      end loop;
    end loop;
    New_Line;
  end Show;

  procedure Show(A1,A2: in Matrix; Cut: in Rep := Zero) is
    use Ada.Text_IO, Txt_Rep_IO;
  begin
    for I in A2'Range(1) loop
      for J in A2'Range(2) loop
        if Abs(A1(I,J))>Cut or Abs(A2(I,J))>Cut then
          Put("(");
          ShortPut(Current_Output,I);
          Put(",");
          ShortPut(Current_Output,J);
          Put(")  ");
          Put(Current_Output,A1(I,J),2,16,4);        -- File,Item,Fore,Aft,Exp
          Put("   ");
          Put(Current_Output,A2(I,J),2,16,4);        -- File,Item,Fore,Aft,Exp
          New_Line;
        end if;
      end loop;
    end loop;
    New_Line;
  end Show;

end Linear.IO;
