with Ada.Text_IO, Reps.Ops;
use Ada.Text_IO, Reps.Ops;

pragma Elaborate_All (Ada.Text_IO, Reps.Ops);

package body Linear.Ops is

--  package Txt_Int_IO is new Ada.Text_IO.Integer_IO(Integer);
--  package Txt_Rep_IO is new Ada.Text_IO.Float_IO(Rep);

  function Norm1(B: Vector) return Rep is
    R: Rep;
  begin
    R := Zero;
    for I in B'Range loop
      R := R+Abs(B(I));
    end loop;
    return R;
  end Norm1;

  function Norm1(Bf,B: Vector) return Rep is
    R: Rep;
  begin
    R := Zero;
    for I in B'Range loop
      R := R+Abs(Bf(I)*B(I));
    end loop;
    return R;
  end Norm1;

  function Norm1(A: Matrix) return Rep is
    R,Rj: Rep;
  begin
    R := Zero;
    for J in A'Range(1) loop
      Rj := Zero;
      for I in A'Range(2) loop
        Rj := Rj+Abs(A(I,J));
      end loop;
      if Rj>R then
        R := Rj;
      end if;
    end loop;
    return R;
  end Norm1;

  function Norm1(Bf: Vector; A: Matrix) return Rep is
    R,Rj: Rep;
  begin
    R := Zero;
    for J in A'Range(1) loop
      Rj := Zero;
      for I in A'Range(2) loop
        Rj := Rj+Abs(Bf(I)*A(I,J));
      end loop;
      R := Rep'Max(R,Rj/Bf(J));
    end loop;
    return R;
  end Norm1;

  function Norm2(B: Vector) return Rep is
    R: Rep;
  begin
    R := Zero;
    for I in B'Range loop
      R := R+B(I)*B(I);
    end loop;
    return Sqrt(R);
  end Norm2;

  procedure Compo(B1,B2,B3: in Vector; R1,R2: out Rep) is
    A,B,D,Det,W1,W2: Rep;
  begin
    A := B1*B1;
    B := B1*B2;
    D := B2*B2;
    Det := A*D-B*B;
    W1 := B3*B1;
    W2 := B3*B2;
    R1 := (D*W1-B*W2)/Det;
    R2 := (A*W2-B*W1)/Det;
  end Compo;

  procedure Add(B1: in Vector; B2: in out Vector) is
  begin
    for I in B2'Range loop
      B2(I) := B2(I)+B1(I);
    end loop;
  end Add;

  procedure Add(A1: in Matrix; A2: in out Matrix) is
  begin
    for I in A2'Range(1) loop
      for J in A2'Range(2) loop
         A2(I,J) := A2(I,J)+A1(I,J);
      end loop;
    end loop;
  end Add;

  procedure Add(R: in Rep; A: in out Matrix) is
  begin
    for I in A'Range(1) loop
      A(I,I) := A(I,I)+R;
    end loop;
  end Add;

  procedure Sub(B1: in Vector; B2: in out Vector) is
  begin
    for I in B2'Range loop
      B2(I) := B2(I)-B1(I);
    end loop;
  end Sub;

  procedure Sub(A1: in Matrix; A2: in out Matrix) is
  begin
    for I in A2'Range(1) loop
      for J in A2'Range(2) loop
         A2(I,J) := A2(I,J)-A1(I,J);
      end loop;
    end loop;
  end Sub;

  procedure Mult(R: in Rep; B: in out Vector) is
  begin
    for I in B'Range loop
      B(I) := R*B(I);
    end loop;
  end Mult;

  procedure Mult(R: in Rep; A: in out Matrix) is
  begin
    for I in A'Range(1) loop
      for J in A'Range(2) loop
        A(I,J) := R*A(I,J);
      end loop;
    end loop;
  end Mult;

  procedure Mult(A1: in Matrix; A2: in out Matrix) is
    R: Rep;
    B: Vector(A2'Range(1));
  begin
    for J in B'Range loop
      for K in B'Range loop
        B(K) := A2(K,J);
      end loop;
      for I in B'Range loop
        R := Zero;
        for K in B'Range loop
          R := R+A1(I,K)*B(K);
        end loop;
        A2(I,J) := R;
      end loop;
    end loop;
  end Mult;

  procedure DMult(Bf: in Vector; B: in out Vector) is
  begin
    for I in B'Range loop
      B(I) := B(I)*Bf(I);
    end loop;
  end DMult;

  procedure InvDMult(Bf: in Vector; B: in out Vector) is
  begin
    for I in B'Range loop
      B(I) := B(I)/Bf(I);
    end loop;
  end InvDMult;

  procedure Cutoff(Rf: in Rep; Bf,B1: in Vector; N1,N2: out Integer; B2: out Vector) is
  begin
    N1 := 0;
    N2 := 0;
    for I in B1'Range loop
      if Abs(B1(I))>Rf*Bf(I) then
        B2(I) := B1(I);
        N1 := N1+1;
        N2 := N2+1;
      else
        B2(I) := Zero;
        if B1(I) /= Zero then
          N1 := N1+1;
        end if;
      end if;
    end loop;
  end Cutoff;

  function "-"(B: Vector) return Vector is
    Bt: Vector(B'Range);
  begin
    for I in B'Range loop
      Bt(I) := -B(I);
    end loop;
    return Bt;
   end "-";

  function "+"(B1,B2: Vector) return Vector is
    B: Vector(B2'Range);
  begin
    for I in B'Range loop
      B(I) := B2(I)+B1(I);
    end loop;
    return B;
  end "+";

  function "+"(A1,A2: in Matrix) return Matrix is
    A: Matrix(A2'Range(1),A2'Range(2));
  begin
    for I in A'Range loop
      for J in A'Range loop
         A(I,J) := A2(I,J)+A1(I,J);
      end loop;
    end loop;
    return A;
  end "+";

  function "-"(B1,B2: Vector) return Vector is
    B: Vector(B2'Range);
  begin
    for I in B'Range loop
      B(I) := B2(I)-B1(I);
    end loop;
    return B;
  end "-";

  function "-"(A1,A2: in Matrix) return Matrix is
    A: Matrix(A2'Range(1),A2'Range(2));
  begin
    for I in A'Range loop
      for J in A'Range loop
         A(I,J) := A2(I,J)-A1(I,J);
      end loop;
    end loop;
    return A;
  end "-";

  function "*"(B1,B2: Vector) return Rep is
    R: Rep;
  begin
    R := Zero;
    for I in B2'Range loop
      R := R+B1(I)*B2(I);
    end loop;
    return R;
  end "*";

  function "*"(R: Rep; B: Vector) return Vector is
    Br: Vector(B'Range);
  begin
    for I in B'Range loop
      Br(I) := R*B(I);
    end loop;
    return Br;
  end "*";

  function "*"(R: Rep; A: Matrix) return Matrix is
    Ar: Matrix(A'Range(1),A'Range(2));
  begin
    for I in A'Range(1) loop
      for J in A'Range(2) loop
        Ar(I,J) := R*A(I,J);
      end loop;
    end loop;
    return Ar;
  end "*";

  function "*"(B1,B2: Vector) return Vector is
    B: Vector(B2'Range);
  begin
    for I in B2'Range loop
      B(I) := B1(I)*B2(I);
    end loop;
    return B;
  end "*";

  function "*"(A: Matrix; B: Vector) return Vector is
    R: Rep;
    Br: Vector(A'Range(1));
  begin
    for I in Br'Range loop
      R := Zero;
      for J in B'Range loop
        R := R+A(I,J)*B(J);
      end loop;
      Br(I) := R;
    end loop;
    return Br;
  end "*";

  function "*"(B: Vector; A: Matrix) return Vector is
    R: Rep;
    Br: Vector(A'Range(2));
  begin
    for I in Br'Range loop
      R := Zero;
      for J in B'Range loop
        R := R+B(J)*A(J,I);
      end loop;
      Br(I) := R;
    end loop;
    return Br;
  end "*";

  function "*"(A1,A2: Matrix) return Matrix is
    R: Rep;
    A: Matrix(A1'Range(1),A2'Range(2));
  begin
    for I in A'Range(1) loop
      for J in A'Range(2) loop
        R := Zero;
        for K in A2'Range(1) loop
          R := R+A1(I,K)*A2(K,J);
        end loop;
        A(I,J) := R;
      end loop;
    end loop;
    return A;
  end "*";

  procedure Balanc(A: in out Matrix) is
 -- translated from numerical recipes
    NearOne: constant Rep := 0.95;
    Radix: constant Rep := Two;
    Done: Boolean;
    N0,N: Integer;
    S,R,G,F,C,Sqrdx: Rep;
  begin
    N0 := A'First(1);
    N := A'Last(1);
    Sqrdx := Radix*Radix;
    loop
      Done := True;
      for I in N0 .. N loop
        C := Zero;
        R := Zero;
        for J in N0 .. N loop
          if J /= I then
            C := C+Abs(A(J,I));
            R := R+Abs(A(I,J));
          end if;
        end loop;
        if C /= Zero and then R /= Zero then
          G := R/Radix;
          F := One;
          S := C+R;
          while C < G loop
            F := F*Radix;
            C := C*Sqrdx;
          end loop;
          G := R*Radix;
          while C > G loop
            F := F/Radix;
            C := C/Sqrdx;
          end loop;
          if (C+R)/F < NearOne*S then
            Done := False;
            G := One/F;
            for J in N0 .. N loop A(I,J) := A(I,J)*G; end loop;
            for J in N0 .. N loop A(J,I) := A(J,I)*F; end loop;
          end if;
        end if;
      end loop;
      exit when Done;
    end loop;
  end Balanc;

  procedure Decomp(A: in out Matrix; Cond: out Rep; Pvt: out Pivot; Work: out Vector) is
 -- translated from numerical recipes
    N0,N,M: Integer;
    T,Anorm,Ynorm,Znorm: Rep;
  begin
    N0 := Work'First;
    N := Work'Last;
    Pvt(N) := N0;

    if N = N0 then
      Cond := One;
      if A(N0,N0) = Zero then
        Put("Singular matrix in Decomp");
        raise Matrix_Singular;
      end if;
      return;
    end if;
    Anorm := Zero;
    for J in N0 .. N loop
      T := Zero;
      for I in N0 .. N loop
        T := T+Abs(A(I,J));
      end loop;
      if T > Anorm then
        Anorm := T;
      end if;
    end loop;
    for K in N0 .. N-1 loop
      M := K;
      for I in K+1 .. N loop
        if Abs(A(I,K)) > Abs(A(M,K)) then
          M := I;
        end if;
      end loop;
      Pvt(K) := M;
      if M /= K then
        Pvt(N) := -Pvt(N);
      end if;
      T := A(M,K);
      A(M,K) := A(K,K);
      A(K,K) := T;
      if T /= Zero then
        for I in K+1 .. N loop
          A(I,K) := -A(I,K)/T;
        end loop;
        for J in K+1 .. N loop
          T := A(M,J);
          A(M,J) := A(K,J);
          A(K,J) := T;
          if T /= Zero then
            for I in K+1 .. N loop
              A(I,J) := A(I,J)+A(I,K)*T;
            end loop;
          end if;
        end loop;
      end if;
    end loop;
    for K in N0 .. N loop
      T := Zero;
      for I in N0 .. K-1 loop
        T := T+A(I,K)*Work(I);
      end loop;
      if A(K,K) = Zero then
        Put("Singular matrix in Decomp");
        raise Matrix_Singular;
        return;
      end if;
      if T >= Zero then
        Work(K) := -(T+One)/A(K,K);
      else
        Work(K) := -(T-One)/A(K,K);
      end if;
    end loop;
    for K in reverse N0 .. N-1 loop
      T := Zero;
      for I in K+1 .. N loop
        T := T+A(I,K)*Work(K);
      end loop;
      Work(K) := T;
      M := Pvt(K);
      if M /= K then
        T := Work(M);
        Work(M) := Work(K);
        Work(K) := T;
      end if;
    end loop;
    Ynorm := Zero;
    for I in N0 .. N loop
      Ynorm := Ynorm+Abs(Work(I));
    end loop;
    Solve(A,Work,Pvt);
    Znorm := Zero;
    for I in N0 .. N loop
      Znorm := Znorm+Abs(Work(I));
    end loop;
    Cond := Anorm*Znorm/Ynorm;
    if Cond < One then
      Cond := One;
    end if;
  end Decomp;

  procedure Solve(A: in Matrix; B: in out Vector; Pvt: in Pivot) is
 -- solve A*B2=B1
 -- input:   A     triangularized matrix obtained from Decomp
 --          Pvt   pivot vector obtained from Decomp
 --          B     vector B1
 -- output:  B     vector B2
    N0,N,M: Integer;
    T: Rep;
  begin
    N0 := B'First;
    N := B'Last;
    for K in N0 .. N-1 loop
      M := Pvt(K);
      T := B(M);
      B(M) := B(K);
      B(K) := T;
      for I in K+1 .. N loop
        B(I) := B(I)+A(I,K)*T;
      end loop;
    end loop;
    for K in reverse N0+1 .. N loop
      B(K) := B(K)/A(K,K);
      T := -B(K);
      for I in N0 .. K-1 loop
        B(I) := B(I)+A(I,K)*T;
      end loop;
    end loop;
    B(N0) := B(N0)/A(N0,N0);
  end Solve;

  function Prod(B0,B1,B2: Vector) return Rep is
    R: Rep;
  begin
    R := Zero;
    for I in B0'Range loop
      R := R+(B0(I)*B0(I))*B1(I)*B2(I);
    end loop;
    for I in B0'Last+1 .. B2'Last loop
      R := R+B1(I)*B2(I);
    end loop;
    return R;
  end Prod;

  function PlusMinusDet(A: Matrix) return Rep is
    C: Rep;
    Pvt: Pivot(A'Range(1));
    B: Vector(A'Range(1));
    Aux: Matrix(A'Range(1),A'Range(2));
   begin
    Aux := A;
    Decomp(Aux,C,Pvt,B);
    C := One;
    for J in A'Range(2) loop
      C := C*Aux(J,J);
    end loop;
    return C;
  end PlusMinusDet;

  procedure Invert(A: in out Matrix) is
    pragma Suppress(Storage_Check);
    C: Rep;
    Pvt: Pivot(A'Range(1));
    B: Vector(A'Range(1));
    Aux: Matrix(A'Range(1),A'Range(2));
  begin
    Aux := A;
    Decomp(Aux,C,Pvt,B);
    for J in A'Range(2) loop
      SetZero(B);
      B(J) := One;
      Solve(Aux,B,Pvt);
      SetColumn(J,B,A);
    end loop;
  end Invert;

  procedure ElmHes(A: in out Matrix) is
 -- ugly, translated from numerical recipes
    N0,N,L: Integer;
    X,Y: Rep;
  begin
    N0 := A'First(1);
    N := A'Last(1);
    if N-N0<2 then return; end if;
    for M in N0+1 .. N-1 loop
      X := Zero;
      L := M;
      for J in M .. N loop
        if Abs(A(J,M-1)) > Abs(X) then
          X := A(J,M-1);
          L := J;
        end if;
      end loop;
      if L /= M then
        for J in M-1 .. N loop
          Y := A(L,J);
          A(L,J) := A(M,J);
          A(M,J) := Y;
        end loop;
        for J in N0 .. N loop
           Y := A(J,L);
           A(J,L) := A(J,M);
           A(J,M) := Y;
        end loop;
      end if;
      if X /= Zero then
        for I in M+1 .. N loop
          Y := A(I,M-1);
          if Y /= Zero then
            Y := Y/X;
            A(I,M-1) := Y;
            for J in M .. N loop A(I,J) := A(I,J)-Y*A(M,J); end loop;
            for J in N0 .. N loop A(J,M) := A(J,M)+Y*A(J,I); end loop;
          end if;
        end loop;
      end if;
    end loop;
  end ElmHes;

  procedure Hqr(A: in out Matrix; Wr,Wi: out Vector) is
 -- ugly, translated from numerical recipes

    function Sign(R,S: Rep) return Rep is
    begin
      if S<Zero then
        return -Abs(R);
      else
        return Abs(R);
      end if;
    end Sign;

    C1: constant Rep := 0.75;
    C2: constant Rep := -0.4375;
    FoundRoot: Boolean;
    N0,N,Nn,M,L,Its: Integer;
    Z,Y,X,W,V,U,T,S,R,Q,P,Anorm: Rep;
  begin
    N0 := A'First(1);
    N := A'Last(1);
    Anorm := Abs(A(N0,N0));
    for I in N0+1 .. N loop
      for J in I-1 .. N loop
         Anorm := Anorm+Abs(A(I,J));
      end loop;
    end loop;
    Nn := N;
    T := Zero;
    while Nn>=N0 loop
      FoundRoot := False;
      Its := 0;
      while not FoundRoot loop
        L := Nn;
        while L>N0 loop
          S := Abs(A(L-1,L-1))+Abs(A(L,L));
          if S=Zero then
            S := Anorm;
          end if;
          exit when (Abs(A(L,L-1))+S) = S;
          L := L-1;
        end loop;
        X := A(Nn,Nn);
        if L=Nn then
          FoundRoot := True;
          Wr(Nn) := X+T;
          Wi(Nn) := Zero;
          Nn := Nn-1;
        else
          Y := A(Nn-1,Nn-1);
          W := A(Nn,Nn-1)*A(Nn-1,Nn);
          if L = Nn-1 then
            FoundRoot := True;
            P := Half*(Y-X);
            Q := P*P+W;
            Z := Sqrt(Abs(Q));
            X := X+T;
            if Q >= Zero then
              Z := P+Sign(Z,P);
              Wr(Nn) := X+Z;
              Wr(Nn-1) := Wr(Nn);
              if Z /= Zero then Wr(Nn) := X-W/Z; end if;
              Wi(Nn) := Zero;
              Wi(Nn-1) := Zero;
            else
              Wr(Nn) := X+P;
              Wr(Nn-1) := Wr(Nn);
              Wi(Nn) := Z;
              Wi(Nn-1) := -Z;
            end if;
            Nn := Nn-2;
          else
            if Its=10 or Its=20 then
              T := T+X;
              for I in N0 .. Nn loop
                A(I,I) := A(I,I)-X;
              end loop;
              S := Abs(A(Nn,Nn-1))+Abs(A(Nn-1,Nn-2));
              X := C1*S;
              Y := X;
              W := C2*S*S;
            elsif Its = 30 then
              Put("Too many iterations in Hqr");
              raise Getting_Tired;
            end if;
            Its := Its+1;
            M := Nn-2;
            if M>=L then
              loop
                Z := A(M,M);
                R := X-Z;
                S := Y-Z;
                P := (R*S-W)/A(M+1,M)+A(M,M+1);
                Q := A(M+1,M+1)-Z-R-S;
                R := A(M+2,M+1);
                S := Abs(P)+Abs(Q)+Abs(R);
                P := P/S;
                Q := Q/S;
                R := R/S;
                exit when M=L;
                U := Abs(A(M,M-1))*(Abs(Q)+Abs(R));
                V := Abs(P)*(Abs(A(M-1,M-1))+Abs(Z)+Abs(A(M+1,M+1)));
                exit when U+V=V;
                M := M-1;
              end loop;
            end if;
            for I in M+2 .. Nn loop
              A(I,I-2) := Zero;
              if  I /= (M+2) then A(I,I-3) := Zero; end if;
            end loop;
            for K in M .. Nn-1 loop
              if K /= M then
                P := A(K,K-1);
                Q := A(K+1,K-1);
                R := Zero;
                if K /= (Nn-1) then
                  R := A(K+2,K-1);
                end if;
                X := Abs(P)+Abs(Q)+Abs(R);
                if X /= Zero then
                  P := P/X;
                  Q := Q/X;
                  R := R/X;
                end if;
              end if;
              S := Sign(Sqrt(P*P+Q*Q+R*R),P);
              if S /= Zero then
                if K=M then
                  if L /= M then A(K,K-1) := -A(K,K-1); end if;
                else
                  A(K,K-1) := -S*X;
                end if;
                P := P+S;
                X := P/S;
                Y := Q/S;
                Z := R/S;
                Q := Q/P;
                R := R/P;
                for J in K .. Nn loop
                  P := A(K,J)+Q*A(K+1,J);
                  if K /= (Nn-1) then
                    P := P+R*A(K+2,J);
                    A(K+2,J) := A(K+2,J)-P*Z;
                  end if;
                  A(K+1,J) := A(K+1,J)-P*Y;
                  A(K,J) := A(K,J)-P*X;
                end loop;
                for I in L .. Integer'Min(Nn,K+3) loop
                  P := X*A(I,K)+Y*A(I,K+1);
                  if K /= (Nn-1) then
                    P := P+Z*A(I,K+2);
                    A(I,K+2) := A(I,K+2)-P*R;
                  end if;
                  A(I,K+1) := A(I,K+1)-P*Q;
                  A(I,K) := A(I,K)-P;
                end loop;
              end if;
            end loop;
          end if;
        end if;
      end loop;
    end loop;
  end Hqr;

  procedure EigenValues(A: in Matrix; Br,Bi: out Vector) is
    Aux: Matrix(A'Range(1),A'Range(2));
  begin
    Aux := A;
    Balanc(Aux);
    ElmHes(Aux);
    Hqr(Aux,Br,Bi);
  end EigenValues;

  procedure UnitVector(B: in out Vector) is
    C: Rep;
  begin
    C := Norm2(B);
    if C>Zero then
      Mult(One/C,B);
    else
      C := One/Sqrt(Rep(B'Last-B'First+1));
      for I in B'Range loop
        B(I) := C;
      end loop;
    end if;
  end UnitVector;

  procedure EigenVector(A: in Matrix; R: in Rep; B: in out Vector; Eps: in Rep := 1.0E-8) is
    Iter: constant Integer := 20;
    C: Rep;
    Pvt: Pivot(B'Range);
    Bt: Vector(B'Range);
    Aux: Matrix(A'Range(1),A'Range(2));
 begin
    Aux := A;
    Add(-R,Aux);
    Decomp(Aux,C,Pvt,Bt);
    UnitVector(B);
    for I in 1 .. Iter loop
      Bt := B;
      Solve(Aux,B,Pvt);
      C := One/Norm2(B);
      if B*Bt<Zero then C := -C; end if;
      Mult(C,B);
      Sub(B,Bt);
      if Norm2(Bt)<Abs(Eps) then return; end if;
    end loop;
    Put(Iter'Img & " iterations in EigenVector");
    raise Getting_Tired;
  end EigenVector;

begin
  null;

exception
  when Matrix_Singular =>
    New_Line;
  when Getting_Tired =>
    New_Line;

end Linear.Ops;
