
package Linear.Ops is

  pragma Elaborate_Body;

  function Norm1(B: Vector) return Rep;                                      -- Standard L1 norm of B
  function Norm1(Bf,B: Vector) return Rep;                                   -- L1 norm of B with weights Bf
  function Norm1(A: Matrix) return Rep;                                      -- Operator norm of A, for standard L1
  function Norm1(Bf: Vector; A: Matrix) return Rep;                          -- Operator norm of A, for weighted L1
  function Norm2(B: Vector) return Rep;                                      -- L2 norm of B
  procedure Compo(B1,B2,B3: in Vector; R1,R2: out Rep);                      -- Components of B3 in directions B1,B2

  procedure Add(B1: in Vector; B2: in out Vector);                           -- B2 := B2+B1
  procedure Add(A1: in Matrix; A2: in out Matrix);                           -- A2 := A2+A1
  procedure Add(R: in Rep; A: in out Matrix);                                -- A := A+R*Identity
  procedure Sub(B1: in Vector; B2: in out Vector);                           -- B2 := B2-B1
  procedure Sub(A1: in Matrix; A2: in out Matrix);                           -- A2 := A2-A1
  procedure Mult(R: in Rep; B: in out Vector);                               -- B := R*B
  procedure Mult(R: in Rep; A: in out Matrix);                               -- A := R*A
  procedure Mult(A1: in Matrix; A2: in out Matrix);                          -- A2 := A1*A2
  procedure DMult(Bf: in Vector; B: in out Vector);                          -- B := Diag(Bf)*B
  procedure InvDMult(Bf: in Vector; B: in out Vector);                       -- B := (Id/Diag(Bf))*B
  procedure Cutoff(Rf: in Rep; Bf,B1: in Vector;
                   N1,N2: out Integer; B2: out Vector);                      -- B2 := B1 part >= Rf*Bf

  function "-"(B: Vector) return Vector;                                     -- -B
  function "+"(B1,B2: Vector) return Vector;                                 -- B1+B2
  function "+"(A1,A2: in Matrix) return Matrix;                              -- A1+A2
  function "-"(B1,B2: Vector) return Vector;                                 -- B1-B2
  function "-"(A1,A2: in Matrix) return Matrix;                              -- A1-A2
  function "*"(B1,B2: Vector) return Rep;                                    -- L2 inner product <B1,B2>
  function "*"(R: Rep; B: Vector) return Vector;                             -- R*B
  function "*"(R: Rep; A: Matrix) return Matrix;                             -- R*A
  function "*"(B1,B2: Vector) return Vector;                                 -- Diag(B1)*B2
  function "*"(A: Matrix; B: Vector) return Vector;                          -- A*B
  function "*"(B: Vector; A: Matrix) return Vector;                          -- Transpose(A)*B
  function "*"(A1,A2: Matrix) return Matrix;                                 -- A1*A2

  procedure Balanc(A: in out Matrix);                                        -- Balance A, preserves spectrum
  procedure Decomp(A: in out Matrix;
                   Cond: out Rep; Pvt: out Pivot; Work: out Vector);         -- Triangularize A, LU decomposition
  procedure Solve(A: in Matrix; B: in out Vector; Pvt: in Pivot);            -- Solve A*B2=B1 for triangularized A
  function PlusMinusDet(A: Matrix) return Rep;                               -- Return Det(A) or -Det(A)
  procedure Invert(A: in out Matrix);                                        -- A := 1/A
  procedure EigenValues(A: in Matrix; Br,Bi: out Vector);                    -- Eigenvalues of A (real and imag parts)
  procedure EigenVector(A: in Matrix; R: in Rep;
                        B: in out Vector; Eps: in Rep := 1.0E-8);            -- Try to solve (A-R)*B=0

private

  procedure UnitVector(B: in out Vector);                                    -- Possible 0th guess for Eigenvector
  procedure ElmHes(A: in out Matrix);                                        -- Reduce A to Hessenberg form
  procedure Hqr(A: in out Matrix; Wr,Wi: out Vector);                        -- Find Eigenvalues of Hessenberg matrix
  function Prod(B0,B1,B2: Vector) return Rep;                                -- Sum of B0(I)*B1(I)*B2(I)

end Linear.Ops;

