with Messages,Parameters;
use Messages,Parameters;

pragma Elaborate_All (Messages);

package body Qs.Ops is

   ScalZero:  constant Scalar := Scal(Zero);
   Trunc: constant Boolean := IsNumeric(ScalZero);

   function Comp(N: Freq; H: Fourier) return Scalar renames Components.Component;
   function ErrComp(N: Freq; H: Fourier) return Scalar renames Components.ErrComponent;
   procedure SetComp(N: Freq; S: in Scalar; H: in out Fourier) renames Components.SetComponent;
   procedure SetErrComp(N: FreqE; S: in Scalar; H: in out Fourier) renames Components.SetErrComponent;
   function Pak(N: Freq; M: Part) return Positive renames Pack;
   procedure UnPak(K: in Positive; N: out Freq; M: out Part) renames UnPack;
   pragma Inline (Pak,UnPak,Comp,ErrComp,SetComp,SetErrComp);

  function "+"(H1,H2: Q) return Q is
     H: Q;
  begin
     if H1.F /= H2.F then
        Message("Qs.Ops.Add",Parity_Error);
     end if;
     H.F := H1.F;
     for K in Part loop
        H.C(K) := H1.C(K)+H2.C(K);
     end loop;
     return H;
  end "+";

  function "-"(H1,H2: Q) return Q is
     H: Q;
  begin
     if H1.F /= H2.F then
        Message("Qs.Ops.Subtract",Parity_Error);
     end if;
     H.F := H1.F;
     for K in Part loop
        H.C(K) := H1.C(K)-H2.C(K);
     end loop;
     return H;
  end "-";

  function "*"(H1,H2: Q) return Q is
    H: Q;
  begin
     if H1.F /= H2.F then
        Message("Qs.Ops.Multiply",Parity_Error);
     end if;
     H.F := H1.F;
     for K in Part loop
        H.C(K) := H1.C(K)*H2.C(K);
     end loop;
     return H;
  end "*";

  function "*"(S: Scalar; H1: Q) return Q is
     H: Q;
  begin
     H.F := H1.F;
     for K in Part loop
        H.C(K) := S*H1.C(K);
     end loop;
     return H;
  end "*";

  function "*"(A: Matrix; H: Q) return Q is
---  all errors of the result are in the coefficients
     N1,N2,Nmax: Freq;
     M1,M2: Part;
     K: Integer;
     C,S: Scalar;
     Wi: array(Freq) of Scalar;
     G: Q;
  begin
--   TraceEnter("Qs.Ops.Prod");
     if not Trunc then
        Wi(0) := Scal(One);
        Wi(1) := Exp(Scal(-Rho(H)));
        K := 1;
        for N in 2 .. Size loop
           Wi(N) := Wi(N-K)*Wi(K);               -- powers of 1/Rho(H)
           if 2*K=N then K := N; end if;
        end loop;
     end if;

     SetZero(Par(H),Rho(H),H.F,G);
     UnPak(A'Last(2),Nmax,M2);

     for K1 in A'Range(1) loop
        UnPak(K1,N1,M1);
        C := ScalZero;
        for K2 in A'Range(2) loop                 -- coeff -> coeff
           UnPak(K2,N2,M2);
           C := C+A(K1,K2)*Comp(N2,H.C(M2));
        end loop;
        if not Trunc then                         -- errors -> coeff
           for M in Part loop
              S := ScalZero;
              for N in reverse Par(H) .. Nmax loop
                 K := Pak(N,M);                   -- should be in A'Range(2)
                 S := Max(S,Abs(A(K1,K))*Wi(N));
                 C := C+S*Ball(ErrComp(N,H.C(M)));
              end loop;
           end loop;
        end if;
        SetComp(N1,C,G.C(M1));
     end loop;
--   TraceLeave;
     return G;
  end "*";

  function DiffNeighM(G: Q) return Q is
    H: Q;
  begin
     H.F := G.F;
     if Sol = 1 or Sol = 11 then
        H.C(1) := G.C(1)-G.C(1);
     end if;  -- 1:A D H     11:L
     if Sol = 13 then
        H.C(1) := G.C(1);
     end if;  -- K F
     if Sol = 0 then
        H.C(1) := G.C(1)-G.C(Number);
     end if;   -- periodic boundary conditions
     if Sol = 3 or Sol = 4 then
        H.C(1) := Scal(Two)*G.C(1);
     end if; -- 3:B   4:C
     if Sol = 7 or Sol = 8 or Sol = 9 then
        H.C(1) := G.C(1)-G.C(2);
     end if;  -- 7:G I    8:E    9:J
     for K in 2 .. Number loop
        H.C(K) := G.C(K)-G.C(K-1);
     end loop;
     return H;
  end DiffNeighM;

  function DiffNeighP(G: Q) return Q is
    H: Q;
  begin
     H.F := G.F;
     if Sol = 0 then
        H.C(Number) :=  G.C(1)-G.C(Number);
     end if;
     if Sol = 7 then
       H.C(Number) :=  Antisymm(G.C(Number-1))-G.C(Number);
     end if;
     if Sol = 8 then
       H.C(Number) :=  -G.C(Number-1)-G.C(Number);
     end if;
     if Sol = 9 then
       H.C(Number) :=  -G.C(Number);
     end if;
     if Sol = 13 or Sol = 11 then
       H.C(Number) :=  G.C(Number-1)-G.C(Number);
     end if;
     if Sol = 3 then
        H.C(Number) :=  Antisymm2(G.C(Number))-G.C(Number);
     end if;
     if Sol = 4 then
        H.C(Number) :=  Scal(-Two)*G.C(Number);
     end if;
     if Sol = 1 then
        H.C(Number) :=  Antisymm(G.C(Number))-G.C(Number);
     end if;
     for K in 1 .. Number-1 loop
        H.C(K) := G.C(K+1)-G.C(K);
     end loop;
     return H;
  end DiffNeighP;

  function MDm2(G: Q) return Q is
     H: Q;
  begin
     H.F := G.F;
     for K in Part loop
        H.C(K) := MDm2(G.C(K));
     end loop;
     return H;
  end MDm2;

  function Norm(G: Q) return Scalar is
     S : Scalar := ScalZero;
  begin
     for K in Part loop
        S := Max(S,Norm(G.C(K)));
     end loop;
     return S;
  end Norm;

  function Norm2(G: Q) return Scalar is
     S : Scalar := ScalZero;
  begin
     for K in Part loop
        S := S+Norm2(G.C(K));
     end loop;
     if Sol = 1 or Sol = 3 or Sol = 7 or Sol = 8 then
        S := Scal(4)*S;
     end if;
     if Sol = 10 then
        S := Scal(2)*S;
     end if;
     if Sol = 13 or Sol = 14 then
        S := Scal(8)*S;
     end if;
     if Sol = 7 or Sol = 8 then
           S := S-Scal(2)*Norm2(G.C(1));
           S := S-Scal(2)*Norm2(G.C(9));
     end if;
     if Sol = 14 then
           S := S-Scal(4)*Norm2(G.C(4));
     end if;
     return S;
  end Norm2;

  procedure GuessDomain(H: in Q) is
    R: Rep := 1.0E+200;
    Rk: Rep;
  begin
     for K in Part loop
        GuessRho(H.C(K),Rk);
--        Show("Component " & K'Img & ": guessing |Im z| <",Rk);
        R := Rep'Min(R,Rk);
     end loop;
     Show("H seems analytic for |Im z| <",R);
     Show("H is used for |Im z| <",Rho(H));
  end GuessDomain;

end Qs.Ops;
