with Messages;
use Messages;

package body Qs is

   function Comp(N: Freq; H: Fourier) return Scalar renames Components.Component;
   function ErrComp(N: Freq; H: Fourier) return Scalar renames Components.ErrComponent;
   procedure SetComp(N: Freq; S: in Scalar; H: in out Fourier) renames Components.SetComponent;
   procedure SetErrComp(N: FreqE; S: in Scalar; H: in out Fourier) renames Components.SetErrComponent;
   function Pak(N: Freq; M: Part) return Positive renames Pack;
   procedure UnPak(K: in Positive; N: out Freq; M: out Part) renames UnPack;
   pragma Inline (Pak,UnPak,Comp,ErrComp,SetComp,SetErrComp);

   procedure SetZero(J: in Parity; R: in Radius; F : in Scalar; H: out Q) is
   begin
      SetZero(J,R,H.C(1));
      H.F := F;
      for K in 2 .. Number loop
         H.C(K) := H.C(1);
      end loop;
   end SetZero;

   function Par(H: Q) return Parity is
   begin
      return Par(H.C(1));
   end Par;

   function Rho(H: Q) return Radius is
   begin
      return Rho(H.C(1));
   end Rho;

   procedure SetRho(R: in Radius; H: in out Q) is
   begin
      for K in Part loop
         SetRho(R,H.C(K));
      end loop;
   end SetRho;

   function Component(N: Part; H: Q) return Fourier is
   begin
      return H.C(N);
   end Component;

   function SubComponent(M:Freq; N: Part; H: Q) return Scalar is
   begin
      return Comp(M,H.C(N));
   end SubComponent;

   function ErrComponent(M : FreqE; N: Part; H: Q) return Scalar is
   begin
      return ErrComp(M,H.C(N));
   end ErrComponent;

   procedure SetComponent(N: in Part; F: in Fourier; H: in out Q) is
   begin
      H.C(N) := F;
   end SetComponent;

   procedure SetSubComponent(M:in Freq; N:in Part; S:in Scalar; H: in out Q) is
   begin
      SetComp(M,S,H.C(N));
   end SetSubComponent;

   procedure SetErrComponent(M: FreqE; N: Part; S: in Scalar; H: in out Q) is
   begin
      SetErrComp(M,S,H.C(N));
   end SetErrComponent;

   function Ball(J: Parity; R: Radius; F: Scalar; Eps: Radius) return Q is
      H: Q;
   begin
      SetZero(J,R,F,H);
      for K in Part loop
        SetErrComp(0,Scal(Eps),H.C(K));
      end loop;
      return H;
   end Ball;

   function Pack(N: Freq; M: Part) return Positive is
   begin
      return M+(N-1)*Number;
   end Pack;

   procedure UnPack(K: in Positive; N: out Freq; M: out Part) is
   begin
      M := 1+ (K-1) mod Number;
      N := 1+ (K-1)/Number;
   end UnPack;

   procedure Pack(H: in Q; A: out Vector) is
      N: Freq;
      M: Part;
   begin
      for K in A'Range loop
         UnPak(K,N,M);
         A(K) := Approx(Comp(N,H.C(M)));
      end loop;
   end Pack;

   procedure UnPack(A: in Vector; J: in Parity; R: in Radius;
                    F: in Scalar; H: out Q) is
      N: Freq;
      M: Part;
   begin
      if J=0 then
         Message("Qs.Ops.UnPack",Not_Implemented);
      end if;
      SetZero(J,R,F,H);
      for K in A'Range loop
         UnPak(K,N,M);
         SetComp(N,Scal(A(K)),H.C(M));
      end loop;
   end UnPack;

   function Frequency(H : Q) return Scalar is
   begin
      return H.F;
   end Frequency;

   procedure SetFrequency(S : in Scalar; H : in out Q) is
   begin
      H.F := S;
   end SetFrequency;

end Qs;
