with Reps,Fouriers_Init,Fouriers,Linear;
use Reps,Fouriers_Init,Linear;

generic

  Number,Size: in Positive;
  type Scalar is private;

  with function IsNumeric(S: Scalar) return Boolean is <>;
  with function Scal(K: Integer) return Scalar is <>;
  with function Scal(R: Rep) return Scalar is <>;
  with function Ball(R: Rep) return Scalar is <>;
  with function Ball(S: Scalar) return Scalar is <>;
  with function Approx(S: Scalar) return Rep is <>;
  with function Inf(S: Scalar) return Rep is <>;
  with function Sup(S: Scalar) return Rep is <>;
  with function SupAbs(S: Scalar) return Rep is <>;
  with function "abs"(S: Scalar) return Scalar is <>;
  with function Min(S1,S2: Scalar) return Scalar is <>;
  with function Max(S1,S2: Scalar) return Scalar is <>;
  with function "-"(S: Scalar) return Scalar is <>;
  with function "+"(S,T: Scalar) return Scalar is <>;
  with function "-"(S,T: Scalar) return Scalar is <>;
  with function "*"(R: Rep; S: Scalar) return Scalar is <>;
  with function "/"(S: Scalar; R: Rep) return Scalar is <>;
  with function Sqr(S: Scalar) return Scalar is <>;
  with function "*"(S,T: Scalar) return Scalar is <>;
  with function Inv(S: Scalar) return Scalar is <>;
  with function "/"(S,T: Scalar) return Scalar is <>;
  with function "**"(S: Scalar; I: Integer) return Scalar is <>;
  with function Exp(S: Scalar) return Scalar is <>;
  with function Log(S: Scalar) return Scalar is <>;

package Qs is

  package Components is new Fouriers (Size => Size, Scalar => Scalar);
  use Components;

  subtype Part is Integer range 1 .. Number;
  type Q is private;

  procedure SetZero(J: in Parity; R: in Radius; F : in Scalar; H: out Q);  -- H := 0 with frequency F
  function Par(H: Q) return Parity;                                        -- Parity of H  (0=even 1=odd)
  function Rho(H: Q) return Radius;
  procedure SetRho(R: in Radius; H: in out Q);                             -- set radius on components
  function Component(N: Part; H: Q) return Fourier;                        -- H[N]
  function SubComponent(M : Freq; N: Part; H: Q) return Scalar;            -- Mth component of H[N]
  function ErrComponent(M : FreqE; N: Part; H: Q) return Scalar;           -- Mth error of H[N]
  procedure SetComponent(N: in Part; F: in Fourier; H: in out Q);          -- H[N] := S
  procedure SetSubComponent(M: in Freq; N: in Part; S: in Scalar; H: in out Q);
  procedure SetErrComponent(M: FreqE; N: Part; S: in Scalar; H: in out Q); -- sets error component
  function Ball(J: Parity; R: Radius; F: Scalar; Eps: Radius) return Q;    -- ball of radius Eps
  function Frequency(H : Q) return Scalar;                                 -- returns H.F
  procedure SetFrequency(S : in Scalar; H : in out Q);                     -- sets H.F
  procedure Pack(H: in Q; A: out Vector);
  procedure UnPack(A: in Vector; J: in Parity; R: in Radius; F: in Scalar; H: out Q);
  function Pack(N: Freq; M: Part) return Positive;
  procedure UnPack(K: in Positive; N: out Freq; M: out Part);

private

  type FourVec is array(Integer range <>) of Fourier;

   type Q is
      record
         F: Scalar;           --- frequency
         C: FourVec(Part);    --- coeffs
      end record;

end Qs;
